import { SeparatedArray, SqlColumn } from '.';
export interface Parens {
    leftSpacing: string;
    rightSpacing: string;
}
export declare type Substitutor = (t: SqlBase, stack: SqlBase[]) => SqlBase | undefined;
export declare type Matcher = (t: SqlBase, stack: SqlBase[]) => boolean;
export interface PrettifyOptions {
    keywordCasing?: 'preserve';
}
export declare type SqlTypeDesignator = 'base' | 'type' | 'columnList' | 'extendClause' | 'columnDeclaration' | 'record' | 'query' | 'withQuery' | 'values' | 'withClause' | 'whereClause' | 'orderByExpression' | 'fromClause' | 'offsetClause' | 'orderByClause' | 'havingClause' | 'limitClause' | 'groupByClause' | 'insertClause' | 'replaceClause' | 'partitionedByClause' | 'partitionByClause' | 'clusteredByClause' | 'withPart' | 'joinPart' | 'alias' | 'labeledExpression' | 'betweenPart' | 'likePart' | 'comparison' | 'literal' | 'placeholder' | 'interval' | 'multi' | 'function' | 'case' | 'whenThenPart' | 'column' | 'star' | 'namespace' | 'table' | 'unary' | 'windowSpec';
export declare type KeywordName = 'all' | 'and' | 'as' | 'case' | 'clusteredBy' | 'decorator' | 'direction' | 'else' | 'end' | 'escape' | 'explainPlanFor' | 'extend' | 'filter' | 'from' | 'groupBy' | 'having' | 'insert' | 'interval' | 'into' | 'join' | 'joinType' | 'limit' | 'not' | 'offset' | 'on' | 'op' | 'orderBy' | 'over' | 'overwrite' | 'partitionBy' | 'partitionedBy' | 'plan' | 'replace' | 'row' | 'select' | 'symmetric' | 'then' | 'timestamp' | 'union' | 'values' | 'when' | 'where' | 'with';
export declare type SpaceName = 'final' | 'initial' | 'not' | 'postAnd' | 'postArguments' | 'postArrow' | 'postAs' | 'postCase' | 'postCaseExpression' | 'postClusteredBy' | 'postColumn' | 'postColumnDeclarations' | 'postColumns' | 'postDecorator' | 'postDot' | 'postElse' | 'postEscape' | 'postExplainPlanFor' | 'postExpressions' | 'postExtend' | 'postFilter' | 'postFrom' | 'postGroupBy' | 'postHaving' | 'postInsert' | 'postInsertClause' | 'postInterval' | 'postIntervalValue' | 'postInto' | 'postJoin' | 'postJoinType' | 'postLeftParen' | 'postLimit' | 'postNamespace' | 'postOffset' | 'postOn' | 'postOp' | 'postOrderBy' | 'postOver' | 'postOverwrite' | 'postPartitionBy' | 'postPartitionedBy' | 'postReplace' | 'postReplaceClause' | 'postRow' | 'postSelect' | 'postSymmetric' | 'postTable' | 'postThen' | 'postTimestamp' | 'postUnion' | 'postValues' | 'postWhen' | 'postWhenExpressions' | 'postWhere' | 'postWindowName' | 'postWith' | 'postWithClause' | 'preAlias' | 'preAnd' | 'preArrow' | 'preAs' | 'preClusteredByClause' | 'preColumns' | 'preDirection' | 'preElse' | 'preEnd' | 'preEscape' | 'preExtend' | 'preFilter' | 'preFromClause' | 'preGroupByClause' | 'preHavingClause' | 'preJoin' | 'preLeftParen' | 'preLimitClause' | 'preOffsetClause' | 'preOn' | 'preOp' | 'preOrderByClause' | 'preOver' | 'preOverwrite' | 'prePartitionedByClause' | 'preUnion' | 'preWhereClause';
export interface SqlBaseValue {
    type?: SqlTypeDesignator;
    spacing?: Partial<Record<SpaceName, string>>;
    keywords?: Partial<Record<KeywordName, string>>;
    parens?: readonly Parens[];
}
export declare abstract class SqlBase {
    static type: SqlTypeDesignator;
    static parseSql(input: string | SqlBase): SqlBase;
    static capitalize: (keyword: string) => string;
    static setCapitalization(capitalization: 'upper' | 'lower' | 'title' | 'random'): void;
    static walkSeparatedArray<T extends SqlBase>(a: SeparatedArray<T>, stack: SqlBase[], fn: Substitutor, postorder: boolean): SeparatedArray<T> | undefined;
    static classMap: Record<string, typeof SqlBase>;
    static register(ex: typeof SqlBase): void;
    static getConstructorFor(type: SqlTypeDesignator): typeof SqlBase;
    static fromValue(parameters: SqlBaseValue): any;
    type: SqlTypeDesignator;
    spacing: Record<string, string>;
    keywords: Record<string, string>;
    parens?: readonly Parens[];
    constructor(options: SqlBaseValue, typeOverride: SqlTypeDesignator);
    valueOf(): SqlBaseValue;
    toJSON(): string;
    equals(other: SqlBase | undefined): boolean;
    logicalEquals(other: SqlBase | undefined): boolean;
    getParens(): readonly Parens[];
    changeParens(parens: readonly Parens[]): this;
    addParens(leftSpacing?: string, rightSpacing?: string): this;
    ensureParens(leftSpacing?: string, rightSpacing?: string): this;
    removeOwnParenSpaces(): this;
    resetOwnSpacing(): this;
    resetOwnKeywords(): this;
    normalizeOwnKeywordSpaces(): this;
    clearOwnSeparators(): this;
    getSpace(name: SpaceName, defaultSpace?: string): string;
    changeSpace(name: SpaceName, space: string): this;
    changeSpaces(newSpacing: Partial<Record<SpaceName, string>>): this;
    protected getSpacingWithout(...toRemove: SpaceName[]): {
        [x: string]: string;
    };
    getKeyword(name: KeywordName, defaultKeyword: string, followSpace?: SpaceName, defaultFollowSpace?: string): string;
    protected getKeywordsWithout(...toRemove: KeywordName[]): {
        [x: string]: string;
    };
    protected abstract _toRawString(): string;
    toString(): string;
    walk(fn: Substitutor): SqlBase;
    walkPostorder(fn: Substitutor): SqlBase;
    _walkHelper(stack: SqlBase[], fn: Substitutor, postorder: boolean): SqlBase | undefined;
    _walkInner(_nextStack: SqlBase[], _fn: Substitutor, _postorder: boolean): SqlBase | undefined;
    isPart(): boolean;
    some(fn: Matcher): boolean;
    every(fn: Matcher): boolean;
    collect(fn: Matcher): SqlBase[];
    getColumns(): SqlColumn[];
    getUsedColumnNames(): string[];
    contains(thing: SqlBase): boolean;
    containsColumnName(columnName: string): boolean;
    getFirstColumnName(): string | undefined;
    prettify(options?: PrettifyOptions): SqlBase;
    prettyTrim(maxLength: number): this;
    apply<T>(fn: (self: this) => T): T;
    applyIf<T>(condition: unknown, fn: (self: this) => T): T | this;
    applyForEach<T>(things: readonly T[], fn: (self: this, thing: T, index: number) => this): this;
}
//# sourceMappingURL=sql-base.d.ts.map