import { SqlBase, SqlBaseValue, SqlTypeDesignator, Substitutor } from '../sql-base';
import { SqlExpression } from '../sql-expression';
import { LiteralValue } from '../sql-literal/sql-literal';
export declare type SqlComparisonOp = '=' | '<>' | '<' | '>' | '<=' | '>=' | 'IS' | 'IN' | 'LIKE' | 'BETWEEN';
export declare type SqlComparisonDecorator = 'ALL' | 'ANY';
export declare type SpecialLikeType = 'includes' | 'prefix' | 'postfix' | 'exact';
export interface SqlComparisonValue extends SqlBaseValue {
    op: SqlComparisonOp;
    negated?: boolean;
    lhs: SqlExpression;
    decorator?: SqlComparisonDecorator;
    rhs: SqlBase;
}
export declare class SqlComparison extends SqlExpression {
    static type: SqlTypeDesignator;
    static DEFAULT_NOT_KEYWORD: string;
    static equal(lhs: SqlExpression | LiteralValue, rhs: SqlExpression | LiteralValue): SqlComparison;
    static unequal(lhs: SqlExpression | LiteralValue, rhs: SqlExpression | LiteralValue): SqlComparison;
    static lessThan(lhs: SqlExpression | LiteralValue, rhs: SqlExpression | LiteralValue): SqlComparison;
    static greaterThan(lhs: SqlExpression | LiteralValue, rhs: SqlExpression | LiteralValue): SqlComparison;
    static lessThanOrEqual(lhs: SqlExpression | LiteralValue, rhs: SqlExpression | LiteralValue): SqlComparison;
    static greaterThanOrEqual(lhs: SqlExpression | LiteralValue, rhs: SqlExpression | LiteralValue): SqlComparison;
    static isNull(lhs: SqlExpression | LiteralValue): SqlComparison;
    static isNotNull(lhs: SqlExpression | LiteralValue): SqlComparison;
    static in(lhs: SqlExpression, values: (SqlExpression | LiteralValue)[]): SqlComparison;
    static notIn(lhs: SqlExpression, values: (SqlExpression | LiteralValue)[]): SqlComparison;
    static like(lhs: SqlExpression | string, rhs: SqlExpression | string, escape?: SqlExpression | string): SqlComparison;
    static notLike(lhs: SqlExpression, rhs: SqlExpression | string, escape?: SqlExpression | string): SqlComparison;
    static between(lhs: SqlExpression | LiteralValue, start: SqlExpression | LiteralValue, end: SqlExpression | LiteralValue): SqlComparison;
    static notBetween(lhs: SqlExpression | LiteralValue, start: SqlExpression | LiteralValue, end: SqlExpression | LiteralValue): SqlComparison;
    static betweenSymmetric(lhs: SqlExpression | LiteralValue, start: SqlExpression | LiteralValue, end: SqlExpression | LiteralValue): SqlComparison;
    static notBetweenSymmetric(lhs: SqlExpression | LiteralValue, start: SqlExpression | LiteralValue, end: SqlExpression | LiteralValue): SqlComparison;
    readonly op: SqlComparisonOp;
    readonly negated?: boolean;
    readonly lhs: SqlExpression;
    readonly decorator?: SqlComparisonDecorator;
    readonly rhs: SqlBase;
    constructor(options: SqlComparisonValue);
    valueOf(): SqlComparisonValue;
    protected _toRawString(): string;
    changeLhs(lhs: SqlExpression): this;
    changeRhs(rhs: SqlBase): this;
    negate(): this;
    _walkInner(nextStack: SqlBase[], fn: Substitutor, postorder: boolean): SqlExpression | undefined;
    getLikeMatchPattern(): string | undefined;
    getSpecialLikeType(): SpecialLikeType | undefined;
}
//# sourceMappingURL=sql-comparison.d.ts.map